import enum


class Client(enum.Enum):
    CHROME = "CHROME"
    CHROME_58 = "CHROME_58"
    CHROME_62 = "CHROME_62"
    CHROME_70 = "CHROME_70"
    CHROME_72 = "CHROME_72"
    CHROME_83 = "CHROME_83"

    FIREFOX = "FIREFOX"
    FIREFOX_55 = "FIREFOX_55"
    FIREFOX_56 = "FIREFOX_56"
    FIREFOX_63 = "FIREFOX_63"
    FIREFOX_65 = "FIREFOX_65"

    IOS = "IOS"
    IOS_11_1 = "IOS_11_1"
    IOS_12_1 = "IOS_12_1"
